/*
 * LSU -- local superuser (logging routines)
 *
 * Author:
 *	Matt Bishop
 *	Research Institute for Advanced Computer Science
 *	NASA Ames Research Center
 *	Moffett Field, CA  94035
 *
 *	mab@riacs.edu			ARPA
 *	...!decvax!decwrl!riacs!mab	UUCP
 *	...!ihnp4!ames!riacs!mab	UUCP
 *
 * Copyright (c) 1986.
 */
#include "lsu.h"

/*
 * open the log file
 */
openlog()
{
	char logfile[BUFSIZ];			/* log file name */
        int logfd;				/* log file descriptor */
	int lerr = 0;				/* 1 if error opening log file */

	/*
	 * open the log file, creating it if need be
	 */
	MAKELOG(logfile);
	if ((logfd = open(logfile, O_WRONLY|O_APPEND|O_CREAT, 0600)) < 0)
		lerr = 1;
	CLEARNAME(logfile);
	/*
	 * get the file pointer
	 */
	if (logfd >= 0)
		logfp = fdopen(logfd, "a");
	else
		logfp = NULL;
	/*
	 * if anything went wrong, log the error if possible
	 */
	if (lerr)
		err(L_INFO|F_COND|F_LOG|F_SYS, "log file");
}

/*
 * write a message to the log
 */
logmessage(flag, logmsg)
unsigned int flag;		/* L_YES if successful */
char *logmsg;			/* message to be logged */
{
	char buf[BUFSIZ];		/* used to format log file entry */

	/*
	 * if the log file is closed,
	 * what can you do?
	 */
	if (logfp == NULL)
		return;

	/*
	 * date it and identify the program
	 */
	(void) fprintf(logfp, "%s %s ", progis->lname, date);

	/*
	 * success or failure?
	 */
	switch(flag&L_MASK){
	case L_INFO:	(void) putc('i', logfp);	break;
	case L_YES:	(void) putc('+', logfp);	break;
	case L_NO:	(void) putc('-', logfp);	break;
	case L_CHK:	(void) putc('v', logfp);	break;
	default:	(void) putc('?', logfp);	break;
	}

	/*
	 * tty name
	 */
	(void) fprintf(logfp, " %-7s ", tty);

	/*
	 * identify the user
	 */
	if (curpw.pw_name == NULL || curpw.pw_name[0] == '\0')
		(void) sprintf(buf, "(%d)-", curpw.pw_uid);
	else
		(void) sprintf(buf, "%s-", curpw.pw_name);
	if (towho != NULL)
		(void) strcat(buf, towho);
	else if (newpw.pw_name != NULL)
		(void) strcat(buf, newpw.pw_name);
	(void) fprintf(logfp, "%-17.17s [%05d] - ", buf, stamp);

	/*
	 * give the message
	 */
	(void) fprintf(logfp, "%s", logmsg);
}
