/*
 * LSU -- local superuser header file
 *
 * Author:
 *	Matt Bishop
 *	Research Institute for Advanced Computer Science
 *	NASA Ames Research Center
 *	Moffett Field, CA  94035
 *
 *	mab@riacs.edu			ARPA
 *	...!decvax!decwrl!riacs!mab	UUCP
 *	...!ihnp4!ames!riacs!mab	UUCP
 *
 * Copyright (c) 1986.
 */
#include <ctype.h>
#include <pwd.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#ifdef LINUX
#include <string.h>
#include <stdlib.h>
#include <malloc.h>
#include <time.h>
#include <errno.h>
#endif

/*
 * this should run on several different versions of UNIX(tm)
 * the dependencies are localized into this file
 */
#include "sysdep.h"

/*
 * the important file names (ie, the permission file and the log file
 * names) are constructed in memory just before use and cleared imme-
 * diately after; this prevents people from snooping through the exe-
 * cutable file to find the log or permission file name.  To change
 * these, change them in the Makefile.  Changing
 *	LSUPERM		changes		the permission file for "lsu"
 *	SUPERM		changes		the permission file for "su"
 *	LOG		changes		the log file
 * DO NOT CHANGE THEM HERE as when you change anything else in the
 * file, the Makefile will clobber the changes you make here.  Also,
 * don't delete anything -- you may screw up the way the Makefile works.
 */
#define	MAKESUPERM(buf) {	/* build the permission file name */ \
/* begin SUPERM */\
buf[0] = '.';\
buf[1] = '/';\
buf[2] = '.';\
buf[3] = 's';\
buf[4] = 'u';\
buf[5] = '\0';\
/* end SUPERM */\
			}
#define	MAKELSUPERM(buf) {	/* build the permission file name */ \
/* begin LSUPERM */\
buf[0] = '.';\
buf[1] = '/';\
buf[2] = '.';\
buf[3] = 'l';\
buf[4] = 's';\
buf[5] = 'u';\
buf[6] = '\0';\
/* end LSUPERM */\
			}
#define	MAKELOG(buf)	{	/* build the log file name */ \
/* begin LOG */\
buf[0] = '.';\
buf[1] = '/';\
buf[2] = '.';\
buf[3] = 'l';\
buf[4] = 'o';\
buf[5] = 'g';\
buf[6] = '\0';\
/* end LOG */\
			}
#define CLEARNAME(buf)	{	/* clear the buffer */		\
				register char *bc = buf;	\
				while(*bc)			\
					*bc++ = '\0';		\
			}

/*
 * internal definitions
 */
			/* some universal UNIX goodies */
					/* lower case version of c */
#define lowcase(c)	(isupper((c))?tolower((c)):(c))
#define SZPASSWORD	8		/* max password length (see getpass(3)) */
#define BOGUSPWD	"\001\001\001"	/* illegal output from crypt(3) */
#define bitset(w,b)	(((w)&(b))==(b))
			/* defaults for lsu */
#define LSUCOMM		'#'		/* begins comment lines in LSUPERM */
#define LSUSHELL	"/bin/sh"	/* default shell */
#define LSUUSER		"root"		/* default user to su/lsu to */
#define SUPERUID	0		/* superuser UID */
			/* logging options */
#define L_MASK		0x000f		/* mask for logging options */
#define L_INFO		0x0000		/* no action */
#define L_NO		0x0001		/* don't allow user to lsu */
#define L_YES		0x0002		/* allow user to lsu */
#define L_CHK		0x0004		/* check syntax */
			/* error options */
#define F_MASK		0x0ff0		/* mask for error options */
#define	F_NONE		0x0000		/* no errors */
#define F_FAIL		0x0010		/* ?su is to fail */
#define	F_COND		0x0020		/* fail UNLESS is (n)su to root */
#define F_PERM		0x0040		/* mail perm file message to LSUMAINT */
#define F_LOG		0x0080		/* mail log file message to LSUMAINT */
#define F_SYS		0x0100		/* system error */
			/* errors in perm file lines */
#define E_MASK		0xf000		/* mask for error in perm file options */
#define E_NONE		0x0000		/* no error */
#define	E_USER		0x1000		/* user not listed */
#define E_NEWU		0x2000		/* user can't lsu/su to that new user */
#define E_TTY		0x4000		/* bad tty */
#define E_TIME		0x8000		/* bad time */
			/* abbreviations for legal program names (see runas[]) */
#define	LSU		(runas[0].pname)/* as local super user */
#define	SU		(runas[1].pname)/* as super user */
#define	NSU		(runas[2].pname)/* as super user */
#define	CSU		(runas[3].pname)/* as syntax checker */
			/* used to determine what shell variables to reset */
#define	V_NONE		0x0001		/* never reset this */
#define	V_ALWAYS	0x0002		/* always reset this */
#define	V_LSU		0x0004		/* reset this for lsu */
#define	V_SU		0x0008		/* reset this for su */
#define	V_NSU		0x0010		/* reset this for nsu */
#define	V_CSU		0x0020		/* reset this for csu */
#define	V_ORLOGIN	0x0040		/* reset if login OR another test met */
#define V_ANDLOGIN	0x0080		/* reset if login AND another test met */
			/* useful versions of NULL */
#define	CNULL		((char *) NULL)
#define	PNULL		((struct passwd *) NULL)

/*
 * forward definitions
 */
char *isinlist();		/* determine if a name is in a list */
char *strsave();		/* make a copy of a string in reserved memory */
FILE *chkperm();		/* check and open the permission file */

/*
 * system variables
 */
#ifndef LINUX
extern int errno;		/* system error number */
extern int sys_nerr;		/* number of elements in sys_errlist[] */
extern char *sys_errlist[];	/* list of system error messages */
#endif

/*
 * system library functions
 */
char *crypt();			/* encrypt password */
char *ctime();			/* get time in ASCII format */
char *getpass();		/* get password from user */
struct passwd *getpwnam();	/* get data in password file for name */
struct passwd *getpwuid();	/* get data in password file for uid */
#ifndef LINUX
char *malloc();			/* allocate some memory */
#endif
char *rindex();			/* get the index of the rightmost char */
char *strcat();			/* catenate a string */
char *strcpy();			/* copy a string */
long time();			/* get time in internal format */
char *ttyname();		/* return the name of associated tty */

/*
 * structure to control permissions and such
 */
struct perms {
	char *lname;		/* log name */
	char *pname;		/* program name */
};

/*
 * structure to control resetting of shell environment variables
 */
struct sv {
	char *splate;		/* template to be reset */
	int len;		/* length of the template to be compared */
	char **cval;		/* if a string required, what it is */
	unsigned int vused;	/* when used */
};

/*
 * global variables
 */
extern char *progname;		/* program name */
extern struct perms *progis;	/* what the program runs as */
extern struct perms runas[];	/* legal program names -- MUST be one of these */
extern FILE *logfp;		/* log file pointer */
extern char *date;		/* date of run */
extern char *tty;		/* terminal name or background */
extern char *towho;		/* command line argument */
extern struct passwd curpw;	/* information about who's running the program */
extern struct passwd newpw;	/* information about who you're lsu'ing to */
extern char *shell;		/* shell to be exec'ed */
extern char *exclude[];		/* exclude directories from path if new UID is 0 */
extern struct sv shvar[];	/* list of shell variables to change */
extern int stamp;		/* PID stamp of this process (for logging) */
extern int dash;		/* 1 if a login shell to be used */
extern int mailmesg;		/* 1 if file problem message to be mailed */
extern unsigned int success;	/* 1 if l/n/csu is to succeed */
#ifndef lint
				/*
				 * these are used to keep track
				 * of how and when the binaries
				 * were made
				 */
extern char *lsu_system;	/* what operating system this was made for */
extern char *lsu_version;	/* what version this is */
#endif
