/*
 * LSU -- local superuser
 *
 * Author:
 *	Matt Bishop
 *	Research Institute for Advanced Computer Science
 *	NASA Ames Research Center
 *	Moffett Field, CA  94035
 *
 *	mab@riacs.edu			ARPA
 *	...!decvax!decwrl!riacs!mab	UUCP
 *	...!ihnp4!ames!riacs!mab	UUCP
 *
 * Copyright (c) 1986.
 */
#include "lsu.h"

#ifdef SYSV_TYPE
#	define INIT		register char *sp = instring;
#	define GETC()		(*sp++)
#	define PEEKC()		(*sp)
#	define UNGETC(c)	(--sp)
#	define RETURN(c)	return;
#	define ERROR(c)	regerr(c)
#	include <regexp.h>
#endif
#ifdef BSD4_TYPE
	char *re_comp();	/* compile regular expression */
#endif

/*
 * global variables
 */
static char *pattern;		/* points to typed-in pattern */
#ifdef SYSV_TYPE
static char patcomp[BUFSIZ];	/* buffer for compiled pattern */
#endif

/*
 * smatch -- set up the pattern to be matched; bomb on error
 */
int smatch(pat)
char *pat;			/* pattern to be matched */
{
#ifdef BSD4_TYPE
	char buf[BUFSIZ];	/* buffer for error message */
	register char *p;	/* points to any error message */

	/*
	 * compile the pattern
	 */
	if ((p = re_comp(pat)) != NULL){
		(void) sprintf(buf, "%s: %s\n", pat, p);
		err(L_INFO|F_COND|F_PERM, buf);
	}
#endif
#ifdef SYSV_TYPE
	pattern = pat;
	(void) compile(pat, patcomp, &patcomp[BUFSIZ], '\0');
#endif
}

/*
 * match -- compare a string to the compiled pattern
 */
match(str)
char *str;			/* string to be compared */
{
#ifdef BSD4_TYPE
	char buf[BUFSIZ];	/* buffer for error message */

	/*
	 * compare appropriately
	 */
	switch(re_exec(str)){
	case 1:			/* success */
		return(1);
	case 0:			/* failure */
		return(0);
	default:
		(void) sprintf(buf, "Internal error comparing %s to %s\n",
					str, pattern);
		err(L_INFO|F_COND, buf);
	}
	return(0);
#endif
#ifdef SYSV_TYPE
	/*
	 * compare appropriately
	 */
	return(step(str, patcomp));
#endif
}

#ifdef SYSV_TYPE
/*
 * regerr -- pattern matching error handler
 */
regerr(n)
int n;				/* number of error */
{
	char buf[BUFSIZ];	/* buffer for error message */

	switch(n){
	case 11:
		(void) sprintf(buf, "%s: range endpoint too large", pattern);
		break;
	case 16:
		(void) sprintf(buf, "%s: bad number", pattern);
		break;
	case 25:
		(void) sprintf(buf, "%s: \"\\digit\" out of range", pattern);
		break;
	case 36:
		(void) sprintf(buf, "%s: illegal or missing delimiter", pattern);
		break;
	case 41:
		(void) sprintf(buf, "%s: no remembered search string", pattern);
		break;
	case 42:
		(void) sprintf(buf, "%s: \\( \\) imbalance", pattern);
		break;
	case 43:
		(void) sprintf(buf, "%s: too many \\(", pattern);
		break;
	case 44:
		(void) sprintf(buf, "%s: more than 2 numbers given in \\{ \\}", pattern);
		break;
	case 45:
		(void) sprintf(buf, "%s: } expected after \\", pattern);
		break;
	case 46:
		(void) sprintf(buf, "%s: first number exceeds second in \\{ \\}", pattern);
		break;
	case 49:
		(void) sprintf(buf, "%s: { } imbalance", pattern);
		break;
	case 50:
		(void) sprintf(buf, "%s: regular expression overflow", pattern);
		break;
	default:
		(void) sprintf(buf, "%s: unknown regexp error %d", pattern, n);
		break;
	}
	err(L_INFO|F_COND|F_PERM, buf);
}
#endif
