/*
 * LSU -- local superuser header file
 *
 * Author:
 *	Matt Bishop
 *	Research Institute for Advanced Computer Science
 *	NASA Ames Research Center
 *	Moffett Field, CA  94035
 *
 *	mab@riacs.edu			ARPA
 *	...!decvax!decwrl!riacs!mab	UUCP
 *	...!ihnp4!ames!riacs!mab	UUCP
 *
 * Copyright (c) 1986.
 */
#include "lsu.h"

/*
 * line information; used to print errors
 */
int linect;			/* line number */

/*
 * perms -- verify the user is listed in a consistent log file
 * if not, log the attempt and end
 */
perms(who)
char *who;                      /* login name of new user */
{
	char buf[BUFSIZ];		/* used as a buffer */
	register char *b, *t;		/* used to scan buffer buf[] */
	int nope;			/* > 0 if no permission */
	int valid = 0;			/* 1 if a valid line found */
	int towholist = 0;		/* 1 if ANYBODY lists who */
	char okuser[BUFSIZ];		/* holds list of legal users */
	int ercode = E_USER;		/* error code */
	int notuser = 0;		/* 0 if line's for another user */
	char ebuf[BUFSIZ];		/* error buffer */
	FILE *permfp;			/* lsuperm file pointer */

	/*
	 * check that the log file is root owned and
	 * readable/writable ONLY by the owner (root)
	 * open it for reading and search for the user name
	 */
	if ((permfp = chkperm()) == NULL){
		if (*who == '\0'){
			if (progname == SU || progname == NSU)
				(void) strcpy(who, LSUUSER);
			else
				(void) strcpy(who, curpw.pw_name);
		}
		return;
	}

	/*
	 * now scan the file for the current user (curpw.pw_name) name
	 * note we parse the line even if this isn't the right user,
	 * since we need to see if the person "newpw.pw_name" is in
	 * the list of people to whom any user can lsu/su
	 */
	for(linect = 1; fgets(buf, BUFSIZ, permfp) != NULL; linect++){
		/*
		 * assume there's nothing wrong yet
		 */
		notuser = 0;
		nope = 0;
		/*
		 * eat comment lines (all begin with LSUCOMM)
		 */
		if (buf[0] == LSUCOMM)
			continue;
		/*
		 * skip leading blanks and see if the name
		 * is that of the current user
		 */
		for(b = buf; *b && !isspace(*b); b++);
		if (isspace(*b))
			*b++ = '\0';
		if (strcmp(buf, curpw.pw_name) != 0){
			notuser++;
			nope++;
		}
		else
			ercode = E_NONE;
		/*
		 * got him
		 * save the list of who he can become
		 */
		if (*b != '\0'){
			while(*b && isspace(*b))
				b++;
			t = okuser;
			while(*b && *b != '\t')
				*t++ = *b++;
			*t = '\0';
		}
		/*
		 * now see if he can do it from this terminal
		 */
		while(*b == '\t')
			b++;
		if (*b != '\0' && !isintty(b) && !notuser){
			(void) sprintf(ebuf, "bad tty (line %d)\n", linect);
			err(L_INFO|F_NONE, ebuf);
			nope++;
		}
		while(*b && *b != '\t')
			b++;
		if (*b == '\t')
			b++;
		/*
		 * see if it is the right time
		 */
		if (!isintime(b) && !notuser){
			(void) sprintf(ebuf, "bad time (line %d)\n", linect);
			err(L_INFO|F_NONE, ebuf);
			nope++;
		}
		/*
		 * now check users -- if the first char in who[] is
		 * '\0', it becomes the first one in the list
		 */
		if (who[0] == '\0' && !notuser){
			for(t = who, b = okuser; *b && *b != ','; t++, b++)
				*t = lowcase(*b);
			*t = '\0';
			if (strcmp("any", who) == 0)
				(void) strcpy(who, LSUUSER);
		}
		else if (isinlist(progname == SU ? curpw.pw_name : who, okuser) == NULL &&
				curpw.pw_uid != 0 &&
					strcmp(curpw.pw_name, who) != 0){
			if (!notuser){
				(void) sprintf(ebuf, "%s not newuser (line %d)\n",
						who, linect);
				err(L_INFO|F_NONE, ebuf);
			}
			nope++;
		}
		else
			towholist++;
		/*
		 * see if it's valid
		 */
		if (nope == 0)
			valid++;
	}
	/*
	 * close the permission file
	 */
	(void) fclose(permfp);

	/*
	 * okay, now get the data to actually do the identity substitution
	 */
	if (who[0] == '\0')
		(void) strcpy(who, LSUUSER);
	/*
	 * is he or is he not legitimate?
	 * YES: if root
	 * NO:  if this is lsu and there's not a valid line in the perm file
	 * NO:  if this is su, there's not a valid line in the perm file, AND
	 *	if any line in the permission file controls access to the user
	 *	in "newpw.pw_name".
	 */
	if (curpw.pw_uid != 0 &&
			strcmp(curpw.pw_name, who) != 0 &&
			(progname == LSU && !valid) ||
			((progname == SU || progname == NSU)
						&& !valid && !towholist)){
		if (ercode == E_USER)
			err(L_INFO|F_FAIL, "user not listed in permission file\n");
		success = F_FAIL;
	}
}

getnewuser(who)
char *who;
{
	struct passwd *pw;
	char buf[BUFSIZ];

	if ((pw = getpwnam(who)) == NULL){
		(void) sprintf(buf, "%s not listed in password file\n", who);
		err(L_INFO|F_COND, buf);
		pw = &curpw;
	}
	newpw.pw_name = strsave(pw->pw_name);
	newpw.pw_passwd = strsave(pw->pw_passwd);
	newpw.pw_uid = pw->pw_uid;
	newpw.pw_gid = pw->pw_gid;
	newpw.pw_dir = strsave(pw->pw_dir);
	newpw.pw_shell = strsave(pw->pw_shell);
}

chkpasswd()
{
	if (progname == LSU)
		vfypwd(curpw.pw_passwd);
	else if (progname == SU || progname == NSU)
		vfypwd(newpw.pw_passwd);
}

/*
 * verify the permissions file is owned by UID 0
 * and readable/writable only by that user
 */
FILE *chkperm()
{
	char perm[BUFSIZ];		/* permission file name */
	char buf[BUFSIZ];		/* buffer for error messages */
	struct stat stbuf;		/* used to check owner, mode of perm file */
	struct passwd *tpw;		/* for errors in ownership */
	FILE *permfp = NULL;		/* pointer to permission file */

	/*
	 * stat the file; note we clobber it
	 * as soon as the call is done
	 */
	if (progname == LSU){
		MAKELSUPERM(perm);
	}
	else if (progname == SU || progname == NSU){
		MAKESUPERM(perm);
	}
	if (stat(perm, &stbuf) < 0){
		CLEARNAME(perm);
		err(L_INFO|F_PERM|F_SYS|F_COND, "permission file");
		CLEARNAME(perm);
		return(NULL);
	}
	CLEARNAME(perm);

	/*
	 * if the UID is not 0,
	 * log who owns the file and quit
	 */
	if (stbuf.st_uid != 0){
		/*
		 * get the UID and password file information for this UID
		 * if none, log the error using the UID and die
		 */
		if ((tpw = getpwuid((int) stbuf.st_uid)) == NULL
						|| tpw->pw_name == NULL)
			(void) sprintf(buf,
				"permission file owned by UID %d\n", stbuf.st_uid);
		else
			(void) sprintf(buf, "permission file owned by %s\n",
							tpw->pw_name);
		/*
		 * protect the file
		 */
		protfile();
		/*
		 * log the error
		 */
		err(L_INFO|F_PERM|F_COND, buf);
		return(NULL);
	}

	/*
	 * check the permissions
	 */
	if ((stbuf.st_mode&0777) != 0600){
		/*
		 * say what mode the file is
		 */
		(void) sprintf(buf, "permission file mode is bad (%04o)\n",
							stbuf.st_mode&07777);
		/*
		 * protect the file
		 */
		protfile();
		/*
		 * log the error
		 */
		err(L_INFO|F_PERM|F_COND, buf);
		return(NULL);
	}

	/*
	 * open the permission file
	 */
	if (progname == LSU){
		MAKELSUPERM(perm);
	}
	else if (progname == SU || progname == NSU){
		MAKESUPERM(perm);
	}
	if ((permfp = fopen(perm, "r")) == NULL){
		CLEARNAME(perm);
		err(L_INFO|F_PERM|F_SYS|F_COND, "permission file");
	}
	CLEARNAME(perm);
	
	/*
	 * return the file pointer
	 */
	return(permfp);
}

/*
 * if there is a problem, change the ownership of the access file to
 * UID 0 GID 0 and the mode to 0000 -- that forces a superuser to look
 * at it
 */
protfile()
{
	char perm[BUFSIZ];		/* buffer for protection file name */

	/*
	 * get the file name
	 */
	if (progname == LSU){
		MAKELSUPERM(perm);
	}
	else if (progname == SU || progname == NSU){
		MAKESUPERM(perm);
	}
	/*
	 * change ownership and mode
	 */
	(void) chown(perm, 0, 0);
	(void) chmod(perm, 0000);
	/*
	 * clobber the name at once
	 */
	CLEARNAME(perm);
}

