/*
 * this should run on the following versions of UNIX(tm):
 *	Amdahl UTS (UTS)
 *	AT&T System V (SYSV)
 *	Berkeley Software Distribution Release 4.2 (BSD4_2)
 *	Berkeley Software Distribution Release 4.3 (BSD4_3)
 *	Cray UNIX Operating System (UNICOS)
 *	NAS Processing System Network Build 3 (NPSN3)
 *	Ridge Operating System Release 3.3 (ROS3_3)
 *	Sequent Balance Dynix Release 2.0 (DYNIX2_0)
 *	Silicon Graphics Graphics Library 2, Workstation 2.3 (SGI2_3)
 *	Silicon Graphics Graphics Library 3, Workstation 3.4 (SGI3_4)
 *	Silicon Graphics Graphics Library 3, Workstation 3.5 (SGI3_5)
 *	Sun Microsystems UNIX Release 4.2 (SUN4_2)
 * here are the defines to make them look the same so far
 * as the code is concerned
 *
 * defines specific to lsu:
 * SYSTEM	complete identification of the system
 * LSUMAIL	shell command to mail a letter; must take the user name
 *		as the last argument, ie. "/bin/mail root" -- BE SURE
 *		A FULL PATH NAME IS USED OR ELSE THERE IS A SECURITY
 *		HOLE (namely, Trojan horses)
 * LSUMAINT	who to mail error messages to (typically, an alias like
 *		"lsumaint")
 * LSUPATH	if there is no PATH environment variable for the user
 *		and he tries to lsu to someone with UID 0, this gets
 *		put in his environment; it is a colon-separated list
 *		of directories
 *
 * these handle the specific AT&T and BSD differences:
 * BSD4_TYPE	assume this is like BSD 4
 * SYSV_TYPE	assume this is like AT&T System V
 *
 * Author:
 *	Matt Bishop
 *	Research Institute for Advanced Computer Science
 *	NASA Ames Research Center
 *	Moffett Field, CA  94035
 *
 *	mab@riacs.arpa			ARPA
 *	...!decvax!decwrl!riacs!mab	UUCP
 *	...!ihnp4!ames!riacs!mab	UUCP
 *
 * Copyright (c) 1986.
 */

#define VERSION	"version 2.2, April 21, 1987	 mab@riacs.edu"

#ifdef LINUX
#define SYSTEM "CSIF LINUX Systems"
#define LINUX_TYPE
#endif

/*
 *	Amdahl UTS (UTS)
 */
#ifdef UTS
#	define SYSTEM	"Amdahl UTS"
#	define LSUPATH	"PATH=/usr/local/bin:/bin:/usr/bin:/usr/amdahl:/usr/ucb/bin:/etc:/usr/local/etc:/usr/hosts"
#	define SYSV_TYPE
#endif

/*
 *	AT&T System V (SYSV)
 */
#ifdef SYSV
#	define SYSTEM	"AT&T Unix System V"
#	define SYSV_TYPE
#endif

/*
 *	Berkeley Software Distribution Release 4.2 (BSD4_2)
 */
#ifdef BSD4_2
#	define SYSTEM	"Berkeley Software Distribution Release 4.2"
#	define BSD4_TYPE
#endif

/*
 *	Berkeley Software Distribution Release 4.3 (BSD4_3)
 */
#ifdef BSD4_3
#	define SYSTEM	"Berkeley Software Distribution Release 4.3"
#	define BSD4_TYPE
#endif

/*
 *	Cray UNIX Operating System (UNICOS)
 */
#ifdef UNICOS
#	define SYSTEM	"Cray Unix Operating System"
#	define SYSV_TYPE
	/*
	 * UNICOS does not support getting speed
	 * from the terminal structure, so CBAUD
	 * is not defined.  Fake it here.
	 */
#	define	CBAUD	037
#endif

/*
 *	NAS Processing System Network Build 3 (NPSN3)
 */
#ifdef NPSN3
#	define SYSTEM	"NAS Processing System Network, Build 3"
#	define SYSV_TYPE
#endif

/*
 *	Ridge Operating System Release 3.3 (ROS3_3)
 */
#ifdef ROS3_3
#	define SYSTEM	"Ridge Computer Systems ROS 3.3"
#	define LSUPATH	"PATH=/usr/ucb:/etc:/bin:/usr/bin"
#	define SYSV_TYPE
#endif

/*
 *	Sequent Balance Dynix Release 2.0 (DYNIX2_0)
 */
#ifdef DYNIX2_0
#	define SYSTEM	"Sequent Computer Systems DYNIX 2.0"
#	define LSUMAINT	"mab"
#	define BSD4_TYPE
#	define LOCAL_VARS { "LOGNAME=%s", 8, &(newpw.pw_name), V_ORLOGIN, },
#endif

/*
 *	Silicon Graphics Graphics Library 2, Workstation 2.3 (SGI2_3)
 */
#ifdef SGI2_3
#	define SYSTEM	"Silicon Graphics IRIS GL2.4"
#	define SYSV_TYPE
#endif

/*
 *	Silicon Graphics Graphics Library 3, Workstation 3.4 (SGI3_4)
 */
#ifdef SGI3_4
#	define SYSTEM	"Silicon Graphics IRIS GL3.4"
#	define SYSV_TYPE
#endif

/*
 *	Silicon Graphics Graphics Library 3, Workstation 3.5 (SGI3_5)
 */
#ifdef SGI3_5
#	define SYSTEM	"Silicon Graphics IRIS GL3.5"
#	define SYSV_TYPE
#endif

/*
 *	Sun Microsystems UNIX Release 4.2 (SUN4_2)
 */
#ifdef SUN4_2
#	define SYSTEM	"Sun Microsystems UNIX Release 4.2"
#	define BSD4_TYPE
#endif

/*
 * these encode generic differences between AT&T System ... and
 * Berkeley ... UNIX
 */
#ifdef SYSV_TYPE
	/*
	 * string library functions
	 */
#	define index	strchr	/* find first occurrance of char in string */
#	define rindex	strrchr	/* find last occurrance of char in string */
	/*
	 * file definitions
	 */
#	include <fcntl.h>	/* file opening modes */
	/*
	 * time definitions
	 */
#	include <time.h>	/* time structures */
	/*
	 * these are mainly for lint
	 */
	void exit();		/* exit declared as a void, not int */
	void free();		/* free declared as a void, not int */
	/*
	 * tty structures are COMPLETELY different!
	 */
#	include <termio.h>	/* terminal description info */
	typedef struct termio T_TTY;	/* terminal control structure */
#	define	IO_GTTY	TCGETA	/* get the tty structure via an ioctl */
#	define	TTYISPEED(x)	(((x).c_cflag)&CBAUD)	/* baud rate as index */
#	define	TTYOSPEED(x)	(((x).c_cflag)&CBAUD)	/* baud rate as index */
				/* allowed terminal speeds */
#	define	SPEEDS	0L,50L,75L,110L,135L,150L,200L,300L,600L,\
			1200L,1800L,2400L,4800L,9600L,19200L,56000L,\
			-1L,
	/*
	 * reset these variables
	 */
#	define SYSTEM_VARS						 \
		{ "HOME=%s",		5, &(newpw.pw_dir),  V_NSU|V_LSU, },\
		{ "LOGNAME=%s",		8, &(newpw.pw_name), V_ORLOGIN,   },\
		{ "MAIL=/usr/mail/%s",  5, &(newpw.pw_name), V_ORLOGIN,   },
	/*
	 * some miscellaneous stuff; used only if not previously defined
	 */
#	ifndef MAILPLATE
#		define MAILPLATE "MAIL=/usr/mail/%s"
#	endif
#	ifndef LSUPATH
#		define LSUPATH "PATH=/etc:/bin:/usr/bin"
#	endif
#endif
#ifdef BSD4_TYPE
	/*
	 * file definitions
	 */
#	include <sys/file.h>	/* file opening modes */
	/*
	 * time definitions
	 */
#	include <sys/time.h>	/* time structures */
	/*
	 * tty structures are COMPLETELY different!
	 */
#	include <sys/ioctl.h>	/* terminal description info */
	typedef struct sgttyb T_TTY;	/* terminal control structure */
#	define	IO_GTTY	TIOCGETP	/* get the tty structure via an ioctl */
#	define	TTYISPEED(x)	((x).sg_ispeed)	/* baud rate as index */
#	define	TTYOSPEED(x)	((x).sg_ospeed)	/* baud rate as index */
				/* allowed terminal speeds */
#	define	SPEEDS	0L,50L,75L,110L,135L,150L,200L,300L,600L,\
			1200L,1800L,2400L,4800L,9600L,19200L,56000L,\
			-1L,
	/*
	 * reset these variables
	 */
#	define SYSTEM_VARS	\
		{ "HOME=%s",		      5, &(newpw.pw_dir),  V_ALWAYS,   },\
		{ "MAIL=/usr/spool/mail/%s",  5, &(newpw.pw_name), V_ORLOGIN,  },
	/*
	 * some miscellaneous stuff; used only if not previously defined
	 */
#	ifndef MAILPLATE
#		define MAILPLATE "MAIL=/usr/spool/mail/%s"
#	endif
#	ifndef LSUPATH
#		define LSUPATH "PATH=/usr/ucb:/etc:/bin:/usr/bin"
#	endif
#endif
#ifdef LINUX_TYPE
	/*
	 * file definitions
	 */
#	include <sys/file.h>	/* file opening modes */
	/*
	 * time definitions
	 */
#	include <sys/time.h>	/* time structures */
	/*
	 * tty structures are COMPLETELY different!
	 */
#	include <termio.h>	/* terminal description info */
	typedef struct termio T_TTY;	/* terminal control structure */
#	define	IO_GTTY	TCGETA	/* get the tty structure via an ioctl */
#	define	TTYISPEED(x)	((x).sg_ispeed)	/* baud rate as index */
#	define	TTYOSPEED(x)	((x).sg_ospeed)	/* baud rate as index */
				/* allowed terminal speeds */
#	define	SPEEDS	0L,50L,75L,110L,135L,150L,200L,300L,600L,\
			1200L,1800L,2400L,4800L,9600L,19200L,56000L,\
			-1L,
	/*
	 * reset these variables
	 */
#	define SYSTEM_VARS	\
		{ "HOME=%s",		      5, &(newpw.pw_dir),  V_ALWAYS,   },\
		{ "MAIL=/usr/spool/mail/%s",  5, &(newpw.pw_name), V_ORLOGIN,  },
	/*
	 * some miscellaneous stuff; used only if not previously defined
	 */
#	ifndef MAILPLATE
#		define MAILPLATE "MAIL=/usr/spool/mail/%s"
#	endif
#	ifndef LSUPATH
#		define LSUPATH "PATH=/usr/ucb:/etc:/bin:/usr/bin"
#	endif
#endif

/*
 * this sets up environment variables that are to be reset
 * note SYSTEM_VARS are defined in the system_TYPE section
 * above, and LOCAL_VARS are defined for each machine
 */
#ifndef SYSTEM_VARS
#	define	SYSTEM_VARS
#endif
#ifndef LOCAL_VARS
#	define	LOCAL_VARS
#endif
#define	LSUVARS {\
	{ "SHELL=%s",	6, &(newpw.pw_shell), V_ALWAYS,    },	\
	{ "USER=%s",	5, &(newpw.pw_name),  V_NSU|V_LSU, },	\
	SYSTEM_VARS						\
	LOCAL_VARS						\
	{ NULL,		0, &(newpw.pw_name),  V_NONE,      },	\
	}

/*
 * generic defines; used only if not previously defined
 */
#ifndef LSUMAIL
#	define LSUMAIL	"/bin/mail"
#endif
#ifndef LSUMAINT
#	define LSUMAINT	"lsumaint"
#endif
#ifndef LSUPATH
#	define LSUPATH	"PATH=/etc:/bin:/usr/bin"
#endif
